<?php
/**
 * @version       1.0.0 22.6.2016
 * @author        Davíð og Golíat
 * @package       Jshopping
 * @copyright     Copyright (C) 2016 dg.is. All rights reserved.
 * @license       GNU/GPL
 */
defined('_JEXEC') or die();

class pm_pei extends PaymentRoot {
    function showPaymentForm($params, $pmconfigs) {
        include(dirname(__FILE__)."/paymentform.php");
    }

    function showAdminFormParams($params) {
        foreach ($array_params as $key){
            if (!isset($params[$key])) $params[$key] = '';
        }
        if (!isset($params['testmode'])) $params['testmode'] = 1;
        if (!isset($params['username'])) $params['username'] = "democlient";
        if (!isset($params['secret'])) $params['secret'] = "demosecret";
        if (!isset($params['merchantId'])) $params['merchantId'] = "1";

        $orders = JSFactory::getModel('orders', 'JshoppingModel');
        include(dirname(__FILE__)."/adminparamsform.php");
    }

    function checkTransaction($pmconfigs, $order, $act) {
        $jshopConfig = JSFactory::getConfig();

        if($pmconfigs['testmode']) {
            $merchantId  = 1;
            $order_url   = "https://externalapistaging.pei.is/api/orders";
            $payment_url = "https://gattinstaging.pei.is/";
        } else {
            $merchantId  = $pmconfigs['merchantId'];
            $order_url   = "https://api.pei.is/api/orders";
            $payment_url = "https://gattin.pei.is/";
        }


        $data = json_decode(file_get_contents('php://input'), true);

        $token = $this->authorize($pmconfigs);

        // saveToLog("payment.log", $token);

        // Create http request with the payload and auth token
        $options = array(
            'http' => array(
                'header' => array(
                    "authorization: " . $token,
                    "content-type: application/json"
                ),
                'method' => 'GET'
            )
        );

        $context = stream_context_create($options);
        $response = file_get_contents($order_url . "/" . $data['orderId'], false, $context);
        echo var_dump($response);

        if ($response == FALSE) {
            // saveToLog("no response", $response);
            return array(3, 'no answer from pei', $data['orderId'], $response);
        } else {
            $response = json_decode($response, true);
            // saveToLog("payment.log", $response);

            if ($response['isConfirmed']) {
                saveToLog("payment.log", "Payment has been completed for order: {$order->order_number}");
                return array(1, "", $data['orderId'], $response);
            } else {
                return array(1, "Pending payment", $data['orderId'], $response);
            }
        }
    }

    function authorize($pmconfigs) {
        $data = array(
            'grant_type' => 'client_credentials',
            'scope' => 'externalapi'
        );

        if($pmconfigs['testmode']) {
            $auth_url     = "https://authstaging.pei.is/core/connect/token";
            $clientId     = "democlient";
            $secret       = "demosecret";
        } else {
            $auth_url    = "https://auth.pei.is/core/connect/token";
            $clientId    = $pmconfigs['username'];
            $secret      = $pmconfigs['secret'];
        }

        // Create the payload, the payload consists of the username and secret that the vendor sets
        $auth_payload = base64_encode("{$clientId}:{$secret}");

        // Create the http parameters with the payload and data included
        $options = array(
            'http' => array(
                'header' => array(
                    "authorization: Basic $auth_payload",
                    "Content-type: application/x-www-form-urlencoded"
                ),
                'method' => 'POST',
                'content' => http_build_query($data)
            )
        );

        // Send data to pei and put the response into $response
        $context  = stream_context_create($options);
        $response = file_get_contents($auth_url, false, $context);

        // If there is no response
        if ($response == FALSE) {
            return false;
        } else {
            // Create an array from the json response from server
            $response = json_decode($response);
            // Move onto the order stage
            return $response->token_type . ' ' . $response->access_token;
        }
    }

    function showEndForm($pmconfigs, $order) {
        $jshopConfig = JSFactory::getConfig();
        $pm_method   = $this->getPmMethod();
        //$item_name   = sprintf(_JSHOP_PAYMENT_NUMBER, $order->order_number);
        // $ssn        = unserialize($order->payment_params_data)['ssn'];

        $this->authenticate($pmconfigs, $order);
    }

    function authenticate($pmconfigs, $order) {
        if($pmconfigs['testmode']) {
            $auth_url     = "https://authstaging.pei.is/core/connect/token";
            $clientId     = "democlient";
            $secret       = "demosecret";
        } else {
            $auth_url    = "https://auth.pei.is/core/connect/token";
            $clientId    = $pmconfigs['username'];
            $secret      = $pmconfigs['secret'];
        }

        $auth_payload = base64_encode("{$clientId}:{$secret}");

        $data = array(
            'grant_type' => 'client_credentials',
            'scope' => 'externalapi'
        );

        $options = array(
            'http' => array(
                'header' => array(
                    "authorization: Basic $auth_payload",
                    "Content-type: application/x-www-form-urlencoded"
                ),
                'method' => 'POST',
                'content' => http_build_query($data)
            )
        );

        $context  = stream_context_create($options);
        $response = file_get_contents($auth_url, false, $context);

        if ($response == FALSE) {
            echo "<p>Could not authenticate with Pei</p>";
            // echo "<p> $auth_payload </p>";
            // echo "<p> $response </p>";
            // echo "<p> $auth_url </p>";
            // echo "<p> $clientId </p>";
            // echo "<p> $secret </p>";
            // echo "<p>" . var_dump($data) . "</p>";
            // echo "<p>" . var_dump($options) . "</p>";
            //write_to_p($auth_payload);
            //write_to_p($response);
        } else {
            $response = json_decode($response);
            $this->redirect_to_order_page($pmconfigs, $order, $response->token_type . ' ' . $response->access_token);
        }
    }

    function redirect_to_order_page($pmconfigs, $order, $token) {
        $pm_method = $this->getPmMethod();
        $item_name = sprintf(_JSHOP_PAYMENT_NUMBER, $order->order_number);

        $order_data - unserialize($order);
        $ssn = unserialize($order->payment_params_data)['ssn'];
        if($pmconfigs['testmode']) {
            $merchantId  = 1;
            $order_url   = "https://externalapistaging.pei.is/api/orders";
            $payment_url = "https://gattinstaging.pei.is/";
        } else {
            $merchantId  = $pmconfigs['merchantId'];
            $order_url   = "https://api.pei.is/api/orders";
            $payment_url = "https://gattin.pei.is/";
        }

        $url_fields = array(
            'option' => 'com_jshopping',
            'controller' => 'checkout',
            'task' => 'step7',
            'act' => 'return',
            'order_id' => $order->order_id,
            'order_number' => $order->order_number,
            'js_paymentclass' => $pm_method->payment_class,
            'hash' => $order->order_hash,
        );

        $notify_url_fields = array(
            'option' => 'com_jshopping',
            'controller' => 'checkout',
            'task' => 'step7',
            'act' => 'notify',
            'order_id' => $order->order_id,
            'order_number' => $order->order_number,
            'js_paymentclass' => $pm_method->payment_class,
            'hash' => $order->order_hash,
            'nolang' => 1
        );

        $uri = JURI::getInstance();
        $liveurlhost = $uri->toString(array("scheme",'host', 'port'));
        //        $return = $liveurlhost.SEFLink("index.php?option=com_jshopping&controller=checkout&task=step7&act=notify&js_paymentclass=".$pm_method->payment_class."&order_id={$order->order_id}&order_number={$order->order_number}");
        $return = JURI::root()."index.php?" . http_build_query($url_fields);
        $notify = JURI::root()."index.php?" . http_build_query($notify_url_fields);
        $this->write_to_p($return);
        $cancel_return = $liveurlhost.SEFLink("index.php?option=com_jshopping&controller=checkout&task=step7&act=cancel&js_paymentclass=".$pm_method->payment_class);

        $order->order_total = $this->fixOrderTotal($order);

        $fields = array(
            'merchantId' => $merchantId,
            'amount' => (int)$order->order_total,
            'successReturnUrl' => $return,
            'cancelReturnUrl' => $cancel_return,
            'postbackUrl' => $notify,
            'reference' => $order->order_number,
            'buyer' => array(
                'name' => $order->f_name . " " . $order->l_name,
                'email' => $order->email,
                'mobileNumber' => $order->phone,
                'ssn' => preg_replace("/[^0-9]/", "", $ssn)
            ),
            'items' => array(
                array(
                    'name' => $item_name
                )
            )
        );

        // saveToLog("payment.log", json_encode($fields));
        // saveToLog("order.log", print_r($order, true));

        /*$items = $order->get_items();

        foreach ($items as $item) {
            $productmeta = new WC_Product($item['product_id']);

            $fields['items'][] = array(
                'code' => $productmeta->get_sku(),
                'name' => $item['name'],
                'quantity' => $item['qty'],
                'amount' => $item['line_total'],
                'unitPrice' => $productmeta->get_price()
            );
            }*/

        $options = array(
            'http' => array(
                'header' => array(
                    "authorization: " . $token,
                    "content-type: application/json"
                ),
                'method' => 'POST',
                'content' => json_encode($fields)
            )
        );

        $context  = stream_context_create($options);
        $response = file_get_contents($order_url, false, $context);

        if ($response == FALSE) {
            echo "<p> Error </p>";
            echo var_dump($response);
        } else {
            $response = json_decode($response);
            // saveToLog("payment.log", $response->orderId);
            print _JSHOP_REDIRECT_TO_PAYMENT_PAGE;
            ?>
            <script>
            window.location.replace("<?php echo "{$payment_url}{$response->orderId}"; ?>");
            </script>
            <?php
        }
    }

    function finish($pmconfigs, $order, $rescode, $act) {
        // saveToLog("payment.log", "wut");
    }

    function notifyFinish($pmconfigs, $order, $rescode) {
        // saveToLog("payment.log", "notifyFinish");
    }

    function complete($pmconfigs, $order, $payment) {
        // saveToLog("payment.log", "complete");
        echo "complete";
    }

    function sendOrderEmail($order_id, $manuallysend = 0) {
        $model = JSFactory::getModel('orderMail', 'jshop');
        $model->setData($order_id, $manuallysend);
        return $model->send();
    }

    function write_to_p($text) {
        echo "<p>";
        echo var_dump($text);
        echo "</p>";
    }

    function fixOrderTotal($order){
        $total = $order->order_total;
        if ($order->currency_code_iso=='HUF'){
            $total = round($total);
        }else{
            $total = number_format($total, 2, '.', '');
        }
        return $total;
    }

    function getUrlParams($pmconfigs){
        $params = array();
        $params['order_id'] = $_GET['order_id'];
        $params['order_number'] = $_GET['order_number'];
        $params['hash'] = "";
        $params['checkHash'] = 0;
        $params['checkReturnParams'] = 0;
        return $params;
    }

    function getStatusFromResCode($rescode, $pmconfigs){
        // saveToLog("payment.log", $rescode);
        /*$status = 0;
        $types_status = array(
            0=>0, 
            1=>$pmconfigs['transaction_end_status'], 
            2=>$pmconfigs['transaction_pending_status'], 
            3=>$pmconfigs['transaction_failed_status'], 
            4=>$pmconfigs['transaction_cancel_status'], 
            5=>$pmconfigs['transaction_open_status'], 
            6=>$pmconfigs['transaction_shipping_status'], 
            7=>$pmconfigs['transaction_refunded_status'], 
            8=>$pmconfigs['transaction_confirm_status'], 
            9=>$pmconfigs['transaction_complete_status'], 
            10=>$pmconfigs['transaction_other_status'],
            99=>0
        );
        $status = $types_status[$rescode];
        saveToLog("payment.log", "status $status");
        saveToLog("pmconfigs.log", print_r($pmconfigs, true));*/
        return $rescode;
    }
}
?>
